//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License


#pragma once

#include "IExtension.h"
#include "../Math/IPolyDef.h"

namespace VectorWorks
{
	namespace Extension
	{
		// ----------------------------------------------------------------------------------------------------
		class DYNAMIC_ATTRIBUTE IShapePaneWidgetAccess
		{
		public:
			enum ERealType {
				eRealType_Normal,
				eRealType_Coord,
				eRealType_Angle,
				eRealType_Area,
				eRealType_Volume,
			};

			enum ESubSelButton {
				eSubSelButton_Unknown,
				eSubSelButton_CurrentButtonUp,
				eSubSelButton_CurrentButtonDown,
				eSubSelButton_PrevButtonDown,
				eSubSelButton_PrevButtonUp,
				eSubSelButton_NextButtonDown,
				eSubSelButton_NextButtonUp,
			};

		public:
			virtual						~IShapePaneWidgetAccess()		{}
			virtual IWidgetsProvider*	GetWidgetProvider() = 0;
			// kWidgetLongInt
			virtual Sint32				GetValueInt(Uint32 viewWidget) = 0;
			virtual void				SetValueInt(Uint32 viewWidget, Sint32 value) = 0;
			// kWidgetBoolean
			virtual bool				GetValueBool(Uint32 viewWidget) = 0;
			virtual void				SetValueBool(Uint32 viewWidget, bool value) = 0;
			// kWidgetReal
			// kWidgetCoordDisp
			// kWidgetCoordLocX
			// kWidgetCoordLocY
			// kWidgetAngleDisp
			// kWidgetAreaDisp
			// kWidgetVolumeDisp
			virtual double				GetValueReal(Uint32 viewWidget, ERealType type = eRealType_Normal) = 0;
			virtual void				SetValueReal(Uint32 viewWidget, double value, ERealType type = eRealType_Normal) = 0;
			// kWidgetText
			// kWidgetPopUp
			// kWidgetRadio
			virtual TXString			GetValueString(Uint32 viewWidget) = 0;
			virtual void				SetValueString(Uint32 viewWidget, const TXString& value) = 0;
			// kWidgetDimStdPopUp
			// kWidgetPrecisionPopUp
			virtual long				GetValueLong(Uint32 viewWidget) = 0;
			virtual void				SetValueLong(Uint32 viewWidget, long value) = 0;
			// kWidgetSubSelection
			virtual ESubSelButton		GetCurrentButton() = 0;
			virtual void				SetHighlightPoint(const WorldPt3& pt) = 0;
			virtual void				ClearHighlightPoint() = 0;
			virtual MCObjectHandle		AddHighlightObject(VectorWorks::Math::IPolyDef* polygon3D) = 0;
			virtual void				RemoveHighlightObject(MCObjectHandle hObject) = 0;
		};

		// ----------------------------------------------------------------------------------------------------
		struct SSelectionContext
		{
			bool	fIsHomogMultiSelEdit;
			bool	fIsSingleObjectEdit;
			bool	fIsMultiSelEdit;
			bool	fIsLocked;
			bool	fIsMultipleSelection;
			bool	fIsDimsAndChainsSelection;
		};

		// ----------------------------------------------------------------------------------------------------
		struct SShapePaneWidgetOnWidgetChange
		{
			IShapePaneWidgetAccess*		fWidgetAccess;
			Sint32						fWidgetID;
			Uint32						fViewWidget;
		};

		// ----------------------------------------------------------------------------------------------------
		struct SShapePaneWidgetTransferValue
		{
			bool						fTransferToView;
			MCObjectHandle				fhObject;
			MCObjectHandle				fhFormat;
			MCObjectHandle				fhRecord;
			IShapePaneWidgetAccess*		fWidgetAccess;
			Sint32						fWidgetID;
			Uint32						fViewWidget;
		};

		// ----------------------------------------------------------------------------------------------------
		// {452F49A5-AEC3-4ce7-BFAF-114C569BB4D0}
		static const TSinkIID IID_ProviderShapePane = { 0x452f49a5, 0xaec3, 0x4ce7, { 0xbf, 0xaf, 0x11, 0x4c, 0x56, 0x9b, 0xb4, 0xd0 } };

		// ----------------------------------------------------------------------------------------------------
		class DYNAMIC_ATTRIBUTE IProviderShapePane : public IEventSink
		{
		public:
			virtual void VCOM_CALLTYPE		Init(CodeRefID objectID, IExtendedProps* extProps) = 0;
			virtual void VCOM_CALLTYPE		Activate(MCObjectHandle hObject, const SSelectionContext& selContext) = 0;
			virtual void VCOM_CALLTYPE		Deactivate() = 0;
			virtual void VCOM_CALLTYPE		Update(IWidgetsProvider* widgetProvider) = 0;
			virtual bool VCOM_CALLTYPE		OnWidgetChange(SShapePaneWidgetOnWidgetChange& data, bool& outNeedReset)	{ return true; }
			// return true if the message is handled and no default translation is to be done
			virtual bool VCOM_CALLTYPE		TransferValue(SShapePaneWidgetTransferValue& data)							{ return false; };
		};
	}
}