//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#pragma once

namespace VWFC
{
	namespace PluginSupport
	{
		using namespace VectorWorks::Extension;

		// ------------------------------------------------------------------------------------------------------
		class VWTool_EventSink	: public VCOMImpl<ILegacyEventSink>
		{
		public:
									VWTool_EventSink(IVWUnknown* parent);
			virtual					~VWTool_EventSink();

			long					GetResult() const;
			void					SetResult(long result);

			void					SetUniversalName(const TXString& name);

		public:
			virtual	void			OnInitXProperties(const IToolPropertiesProvider* pPropsProvider);

			virtual void			OnSelectionChange();
			virtual void			OnCustomBar_Setup();
			virtual void			OnCustomBar_Setdown();
			virtual void			OnCustomBar_UpdateValues();
			virtual void			OnCustomBar_LockValue(size_t fieldIndex, const TXString& userString);
			virtual void			OnCustomBar_UnlockValue(size_t fieldIndex);

			// When tool is selected or resumed
			virtual bool			DoSetUp(bool bRestore, const IToolModeBarInitProvider* pModeBarInitProvider);
			// When tool is deselected	or paused
			virtual void			DoSetDown(bool bRestore, const IToolModeBarInitProvider* pModeBarInitProvider);
			virtual void			DoDrawScrMod();
			virtual void			DoUndrawScrMod();

			// init the tool data for creation of the specified object
			virtual void			OnToolInitByObject(MCObjectHandle hObject, const IToolModeBarProvider* pModeBarInitProvider);

			// When tool selected and curs in doc
			virtual void			GetCursorSpecifier(TXString& cursorSpecifier);
			// When app needs tool's status
			virtual TToolStatus		GetStatus(const IToolStatusProvider* pStatusProvider);

			// When mouse is clicked in drawing
			virtual void			PointAdded();
			// When app removes a point
			virtual void			PointRemoved(bool bClearAllPts);
			// Whenever tool needs to draw
			virtual void			Draw(IToolDrawProvider* pDrawProvider);
			// When mouse moves in drawing
			virtual void			MouseMove();
			// When tool is done getting points
			virtual void			HandleComplete() = 0;

			// When mouse is clicked in mode bar
			virtual void			DoModeEvent(size_t modeGroupID, size_t newButtonID, size_t oldButtonID);
			// When tool is double-clicked
			virtual void			DoDoubleClick();
			// When double-click in drawing
			virtual bool			DrawingDoubleClick();
			// After every key event.
			virtual void			GenericStateChange();

			// The default event notificaiotn
			// if no virutal is present for an event, this function is called to allow the user to catch everything
			virtual long			OnDefaultEvent(long action, long message1, long message2);

		// tool points
		public:
			size_t					GetToolPointsCount() const;
			void					ClearAllToolPoints();
			VWPoint2D				GetToolPt2D(size_t ptIndex) const;
			VWPoint3D				GetToolPt3D(size_t ptIndex) const;
			bool					GetToolPtCurren2D(VWPoint2D& outPt) const;
			bool					GetToolPtCurren3D(VWPoint3D& outPt) const;
			bool					PopLastToolPt();

		// hot spot points
		public:
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
			long					HotSpotAdd(HotSpotType type, const VWPoint3D& pt, const TXString& cursorSpec);
			long					HotSpotAdd(HotSpotType type, const VWPoint3D& pt, const TXString&, const TXString& strTip);
#endif
			long					HotSpotAdd(HotSpotType type, const VWPoint3D& pt, long cursorID);
			long					HotSpotAdd(HotSpotType type, const VWPoint3D& pt, long cursorID, const TXString& strTip);

			long					HotSpotTrack(bool& bOutIs3D, VWPoint3D& outLocation);
			void					HotSpotRemove(long hotSpotID);

		// snap geometry
		public:
			long					SnapGeometryAdd(MCObjectHandle hSnapGeometry);
			void					SnapGeometryRemove(long snapGeometryID);

		// help text
		public:
			void					SetModeBarHelpText(const TXString& helpText);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE			LegacyMain(long action, long message1, long message2);

		protected:
			virtual void			Default();

		protected:
			IToolModeBarProvider*	fpModeBarProvider;
			long					fResult;
			TXString				fUnivName;
			TXString				fCustomHelpText;
		};

		namespace ToolDef
		{
			enum EPickAndUpdate		{ disablePickUpdate, pickAndUpdate };
			enum ENeedScreenPlane	{ doesntNeedScreenPlane, needsScreenPlane };
			enum ENeed3DView		{ doesntNeed3DView, needs3DView };
			enum EUseCursor			{ use2DCursor, use3DCursor };
			enum EConstrantCursor	{ constrainCursor, screenBasedCursor };
			enum ENeedPerspective	{ doesntNeedPerspective, needsPerspective };
			enum EShowScreenHints	{ showScreenHints, dontShowScreenHints };
			enum ENeedsPlanarContext	{ needsPlanarContext, doesntNeedPlanarContext };
		}

		// ------------------------------------------------------------------------------------------------------
		struct SToolDef
		{
			EExtensionToolType			fToolType;
			const char*					fParametricName;	// use eExtensionToolType_Normal if you specify parametric name
															// the type will be determined by the type of the parametric
			ToolDef::EPickAndUpdate		fPickAndUpdate;
			ToolDef::ENeedScreenPlane	fNeedScreenPlane;
			ToolDef::ENeed3DView		fNeed3DView;
			ToolDef::EUseCursor			fUseCursor;
			ToolDef::EConstrantCursor	fConstrainCursor;
			ToolDef::ENeedPerspective	fNeedPerspective;
			ToolDef::EShowScreenHints	fShowScreenHints;
			ToolDef::ENeedsPlanarContext	fNeedsPlanarContext;
			bool						fReserved1;
			Sint16						fReserved2;
#if 1 /* T00809_IMAGE_RESOURCE_MANAGER_ENABLED commented out so that sample plugins can have same access */
			Sint16						fReserved3;
			Sint16						fReserved4;
#else
			Sint16						fIconID;
			Sint16						fDefaultCursorID;
#endif
			SSTRResource				fMessage;
			Sint16						fWaitMoveDistance;
			Sint16						fConstraintFlags;
			EToolBarDisplay				fBarDisplay;
			Sint16						fMinimumCompatibleVersion;
			SSTRResource				fTitle;
			SSTRResource				fCategory;
			STEXTResource				fHelpText;
			Uint16						fVersionCreated;
			Uint16						fVersoinModified;
			Uint16						fVersoinRetired;
			const char*					fContextualHelpID;
//#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED /* commented out so that sample plugins can have same access */
			const char*					fIconSpec;
			const char*					fCursorImageSpec;
//#endif
		};


		// ------------------------------------------------------------------------------------------------------
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
		class VWExtensionTool   : public VCOMImmediateImpl<IExtension2Tool>
#else
		class VWExtensionTool	: public VCOMImmediateImpl<IExtensionTool>
#endif
		{
		public:
											VWExtensionTool(	CallBackPtr cbp,
																const SToolDef& toolDef);
			virtual							~VWExtensionTool();

		// IExtension
		public:
			virtual IEventSink* VCOM_CALLTYPE		QueryEventSink(const TSinkIID& iid);

		// IExtensionTool
		public:
			virtual EExtensionToolType				GetType();
			virtual const TXString& VCOM_CALLTYPE	GetTitle();
			virtual const TXString& VCOM_CALLTYPE	GetCategory();
			virtual const TXString& VCOM_CALLTYPE	GetHelpText();
			virtual const TXString& VCOM_CALLTYPE	GetMessage();
			virtual const TXString& VCOM_CALLTYPE	GetContextualHelpID();
			virtual const TXString&	VCOM_CALLTYPE	GetParametricName();
			virtual bool VCOM_CALLTYPE				GetPickAndUpdate();
			virtual bool VCOM_CALLTYPE				GetNeedScreenPlane();
			virtual bool VCOM_CALLTYPE				GetNeed3DProjection();
			virtual bool VCOM_CALLTYPE				GetUse2DCursor();
			virtual bool VCOM_CALLTYPE				GetConstrainCursor();
			virtual bool VCOM_CALLTYPE				GetNeedPerspective();
			virtual bool VCOM_CALLTYPE				GetShowScreenHints();
			virtual bool VCOM_CALLTYPE				GetNeedsPlanarContext();
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
			virtual void VCOM_CALLTYPE				GetIconSpec(TXString& imageSpec);
			virtual void VCOM_CALLTYPE				GetDefaultCursorImageSpec(TXString& imageSpec);
			virtual Sint16 VCOM_CALLTYPE			Deprecated15();
			virtual Sint16 VCOM_CALLTYPE			Deprecated16();
#else
			virtual Sint16 VCOM_CALLTYPE			GetIconID();
			virtual Sint16 VCOM_CALLTYPE			GetDefaultCursorID();
#endif
			virtual Sint16 VCOM_CALLTYPE			GetWaitMoveDistance();
			virtual Sint16 VCOM_CALLTYPE			GetConstraintFlags();
			virtual EToolBarDisplay VCOM_CALLTYPE	GetBarDisplay();
			virtual	Sint16 VCOM_CALLTYPE			GetMinimumCompatibleVersion();
			virtual	void VCOM_CALLTYPE				GetVersionInfo(Uint16& outCreated, Uint16& outModified, Uint16& outRetired);

		protected:
			virtual void						OnRefCountZero();
			virtual ILegacyEventSink*			CreateLegacyEventSink(IVWUnknown* parent) = 0;
			virtual bool						GetProtectionSinkIID(TSinkIID& outIID);
			virtual IProviderProtection*		CreateProtectionSink(IVWUnknown* parent);

		private:
			CallBackPtr						fCBP;

			const SToolDef&					fToolDef;
			TXString						fstrTitle;
			TXString						fstrCategory;
			TXString						fstrHelp;
			TXString						fstrMessage;
			TXString						fstrContextualHelpID;
			TXString						fstrParametricName;

			ILegacyEventSink*				fLegacyEventSink;
			bool							fProtectionObtained;
			bool							fHasProtection;
			TSinkIID						fProtectionSinkIID;
			IProviderProtection*			fProtectionSink;
		};

		// ------------------------------------------------------------------------------------------------------
		// ------------------------------------------------------------------------------------------------------
		// Specific purpose event sinks implementations


		// ------------------------------------------------------------------------------------------------------
		class VWToolEmpty_EventSink	: public VCOMImpl<ILegacyEventSink>
		{
		public:
									VWToolEmpty_EventSink(IVWUnknown* parent);
			virtual					~VWToolEmpty_EventSink();

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE LegacyMain(long action, long message1, long message2);
		};

		// ------------------------------------------------------------------------------------------------------
		class VWToolDefault_EventSink	: public VWTool_EventSink
		{
		public:
									VWToolDefault_EventSink(IVWUnknown* parent, const TXString& objectName);
			virtual					~VWToolDefault_EventSink();

		public:
			// override to modify the fToolInfo. Set 'fUseToolInfo' to true of the info is valid
			virtual void			Init(const TXString& universalName);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE LegacyMain(long action, long message1, long message2);

		// VWTool_EventSink
		public:
			virtual void			Draw(IToolDrawProvider* pDrawProvider);
			virtual void			HandleComplete();

		protected:
			virtual void			Default();

		protected:
			TXString				fObjectName;
			EDefaultToolType		fToolType;
			bool					fIsStandardTool;
			long					fUserData;

			struct SActionData
			{
				long	fAction;
				long	fMessage1;
				long	fMessage2;
			};

			typedef std::vector<SActionData>	TActionDataArray;
			TActionDataArray		fLastActions;

		};

		// ------------------------------------------------------------------------------------------------------
		class VWToolDefaultPoint_EventSink	: public VWToolDefault_EventSink
		{
		public:
									VWToolDefaultPoint_EventSink(IVWUnknown* parent, const TXString& objectName);
			virtual					~VWToolDefaultPoint_EventSink();

		public:
			// override to modify the fToolInfo. Set 'fUseToolInfo' to true of the info is valid
			virtual void					Init(const TXString& universalName);
			const SDefaultPointToolInfo&	GetToolInfo() const;
			void							SetToolInfo(const SDefaultPointToolInfo& info);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE LegacyMain(long action, long message1, long message2);

		private:
			SDefaultPointToolInfo	fToolInfo;
			SDefaultPointToolInfo	fVWToolInfo;
		};

		// ------------------------------------------------------------------------------------------------------
		class VWToolDefaultLine_EventSink	: public VWToolDefault_EventSink
		{
		public:
									VWToolDefaultLine_EventSink(IVWUnknown* parent, const TXString& objectName);
			virtual					~VWToolDefaultLine_EventSink();

		public:
			// override to modify the fToolInfo. Set 'fUseToolInfo' to true of the info is valid
			virtual void				Init(const TXString& universalName);
			const SDefaultLineToolInfo&	GetToolInfo() const;
			void						SetToolInfo(const SDefaultLineToolInfo& info);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE LegacyMain(long action, long message1, long message2);

		private:
			SDefaultLineToolInfo	fToolInfo;
			SDefaultLineToolInfo	fVWToolInfo;
		};

		// ------------------------------------------------------------------------------------------------------
		class VWToolDefaultRect_EventSink	: public VWToolDefault_EventSink
		{
		public:
									VWToolDefaultRect_EventSink(IVWUnknown* parent, const TXString& objectName);
			virtual					~VWToolDefaultRect_EventSink();

		public:
			// override to modify the fToolInfo. Set 'fUseToolInfo' to true of the info is valid
			virtual void				Init(const TXString& universalName);
			const SDefaultRectToolInfo&	GetToolInfo() const;
			void						SetToolInfo(const SDefaultRectToolInfo& info);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE LegacyMain(long action, long message1, long message2);

		private:
			SDefaultRectToolInfo	fToolInfo;
			SDefaultRectToolInfo	fVWToolInfo;
		};

		// ------------------------------------------------------------------------------------------------------
		class VWToolDefault2DPoly_EventSink	: public VWToolDefault_EventSink
		{
		public:
									VWToolDefault2DPoly_EventSink(IVWUnknown* parent, const TXString& objectName);
			virtual					~VWToolDefault2DPoly_EventSink();

		public:
			// override to modify the fToolInfo. Set 'fUseToolInfo' to true of the info is valid
			virtual void					Init(const TXString& universalName);
			const SDefault2DPolyToolInfo&	GetToolInfo() const;
			void							SetToolInfo(const SDefault2DPolyToolInfo& info);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE LegacyMain(long action, long message1, long message2);

		private:
			SDefault2DPolyToolInfo	fToolInfo;
			SDefault2DPolyToolInfo	fVWToolInfo;
		};

		// ------------------------------------------------------------------------------------------------------
		class VWToolDefault3DPoly_EventSink	: public VWToolDefault_EventSink
		{
		public:
									VWToolDefault3DPoly_EventSink(IVWUnknown* parent, const TXString& objectName);
			virtual					~VWToolDefault3DPoly_EventSink();

		public:
			// override to modify the fToolInfo. Set 'fUseToolInfo' to true of the info is valid
			virtual void					Init(const TXString& universalName);
			const SDefault3DPolyToolInfo&	GetToolInfo() const;
			void							SetToolInfo(const SDefault3DPolyToolInfo& info);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE LegacyMain(long action, long message1, long message2);

		private:
			SDefault3DPolyToolInfo	fToolInfo;
			SDefault3DPolyToolInfo	fVWToolInfo;
		};
	}
}
