//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#pragma once


namespace VWFC
{
	namespace PluginSupport
	{
		// ------------------------------------------------------------------------------------------------------
		struct SSTRResource
		{
			short	fListID;
			short	fIndex;

			bool	operator<(const SSTRResource& other) const;
		};

		// ------------------------------------------------------------------------------------------------------
		struct STEXTResource
		{
			short	fID;

			bool	operator<(const STEXTResource& other) const;
		};

		// ------------------------------------------------------------------------------
		enum EObjectEvent {
				kObjectEventNoErr			= kParametricNoErr,			//= 0;		// 
				kObjectEventSetupFailed		= kParametricSetupFailed,	//= -1;
				kObjectEventError			= kParametricError,			//= -2;
				kObjectEventNotImplemented	= kParametricNotImplemented,//= -3;		// The Plug-in did not present its own preference dialog, it is up to the VW application to do it.
				kObjectEventPrefOK			= kParametricPrefOK,		//= -4;		// The Plug-in presented its own preferences dialog, and the user hit OK.
				kObjectEventPrefCancel		= kParametricPrefCancel,	//= -5;		// The Plug-in presented its own preferences dialog, and the user hit Cancel.
				kObjectEventHadError		= kParametricHadError,		//= -6;		// The Plug-in compiled but had an error during execution.
				kObjectEventHadWarning		= kParametricHadWarning,	//= -7;		// The Plug-in compiled but had a warning during execution.
                kObjectEventStayLoaded      = -12,                      //= -12;    // The Plug-in returns this value on InitializeExt in order to stay loaded indefinitely
				kObjectEventUIButtonOK		= kObjectUIButtonHitOK,
				kObjectEventUIButtonCancel	= kObjectUIButtonHitCancel,
		};

		// ------------------------------------------------------------------------------
		enum EMenuCallbackEvent {
			kMenuCallbackEventActiveDocChange	= 'DOCC',		// Send when a different document becomes the frontmost one
			kMenuCallbackEventDocClose			= 'DCLS',		// Send when a document closes.
			kMenuCallbackEventSelChange			= 'SELC',
			kMenuCallbackEventLockObjectChange	= 'Lock',		// Object(s) have had their Lock flag changed.
			kMenuCallbackEventMenuDone			= 'DONE',
			kMenuCallbackEventWPChange			= 'wpch',
			kMenuCallbackEventWPListChange		= 'wpxx',
			kMenuCallbackEventDimStdChange		= 'DIMS',
			kMenuCallbackEventLayerChange		= 'LAYR',
			kMenuCallbackEventClassChange		= 'CLAS',
			kMenuCallbackEventSubscripAddDel	= 'SbAD',		// Send when a subscription [reference] has been added or deleted
			kMenuCallbackEvent3DViewChange		= '3DVC',
			kMenuCallbackEventViewChange		= 'VIEW',
			kMenuCallbackEventTextChange		= 'TEXT',		// Send when text menus or att palette is changed on a VW text object
			kMenuCallbackEventWallStyleChange	= 'WaCh',		// Send when a Wall Style has been added, deleted, or edited
			kMenuCallbackEventWallAdd		 	= 'WAdd',		// Send when a wall or round wall has been added
			kMenuCallbackEventWallDelete	 	= 'WDel',		// Send when a wall or round wall has been deleted
			kMenuCallbackEventWallEdit		 	= 'WEdt'		// Send when a wall or round wall has been edited
		};

		// ---------------------------------------------------------------------------
		enum EStandardCursor {
			kStandardCursor_Arrow			= 147,
			kStandardCursor_ArrowShort		= 140,
			kStandardCursor_ArrowDiagonal	= 107,
			kStandardCursor_ArrowVertical	= 401,
			kStandardCursor_FourArrows		= 93,
			kStandardCursor_HandFinger		= 304,
			kStandardCursor_HandLeft		= 104,
			kStandardCursor_Hand			= 145,
			kStandardCursor_CrossSmall		= 103,
			kStandardCursor_CrossBig		= 105,
			kStandardCursor_CrossBigDashed	= 106,
			kStandardCursor_CrossBezier		= 330,
			kStandardCursor_CrossCubic		= 331,
			kStandardCursor_CrossArc		= 332,
			kStandardCursor_Target			= 154,
			kStandardCursor_Bucket			= 126,
			kStandardCursor_Pipette			= 442,
			kStandardCursor_XCrosshairs		= 159,
		};

		// ---------------------------------------------------------------------------
		typedef		long			TToolStatus;
		class IToolStatusProvider
		{
		public:
			virtual					~IToolStatusProvider() {}
			virtual	TToolStatus		GetOnePointToolStatus() const = 0;
			virtual TToolStatus		GetTwoPointToolStatus() const = 0;
			virtual TToolStatus		GetThreePointToolStatus() const = 0;
			virtual TToolStatus		GetPolyToolStatus() const = 0;
			virtual TToolStatus		GetClickDragToolStatus() const = 0;
		};

		// ---------------------------------------------------------------------------
		class IToolModeBarProvider
		{
		public:
			virtual				~IToolModeBarProvider() {}
			virtual void		EnableModeGroup(size_t modeGroupID, bool bEnable) const = 0;
			virtual void		SetHelpString(const TXString& helpMessage) const = 0;
			virtual SInt32		GetModeGroupValue(size_t modeGroupID) const = 0;
			virtual void		SetModeGroupValue(size_t modeGroupID, SInt32 value) const = 0;
			virtual void		InsertPullDownMenuItem(size_t modeGroupID, const TXString& text) const = 0;
			virtual void		SetPullDownMenuSelection(size_t modeGroupID, size_t selection) const = 0;
			virtual bool		SetButtonsHelpText(const TXStringArray& arrHelpText) const = 0;
		};

		// ---------------------------------------------------------------------------
		class IToolModeBarInitProvider : public IToolModeBarProvider
		{
		public:
			virtual				~IToolModeBarInitProvider() {}
			
			virtual void		AddButtonModeGroup(short ICNRes) const = 0;
			virtual void		AddCheckModeGroup(short ICNRes, bool initState) const = 0;

			virtual void		AddPullDownMenuModeGroup(const TXString& label) const = 0;

			virtual void		AddRadioModeGroup(size_t initState, size_t btnCnt, short id1, short id2, short id3=0, short id4=0, short id5=0, short id6=0 ) const = 0;
			virtual void		AddTextButtonModeGroup(const TXString& text) const = 0;
		};

//T00809_IMAGE_RESOURCE_MANAGER_ENABLED
		class IToolModeBar2InitProvider : public IToolModeBarInitProvider
		{
		public:
			virtual				~IToolModeBar2InitProvider() {}

			virtual void		AddButtonModeGroup2(const TXString& iconSpec) const = 0;
			virtual void		AddCheckModeGroup2(const TXString& iconSpec, bool initState) const = 0;
			virtual void		AddRadioModeGroup2(size_t initState, const TXStringArray& imageSpecs) const = 0;
		};
		
		// ---------------------------------------------------------------------------
		class IToolDrawProvider
		{
		public:
			virtual				~IToolDrawProvider() {}

			// default drawing
			virtual void		DrawDefaultRect() const = 0;
			virtual void		DrawDefault2D() const = 0;
			virtual void		DrawDefault3D() const = 0;

			// simple drawing
			virtual void		DrawLine2D(const VWLine2D& line) const = 0;
			virtual void		DrawLine3D(const VWLine3D& line) const = 0;
			virtual void		DrawRect(const VWRectangle2D& rect) const = 0;
			virtual void		DrawArc(const VWArc2D& arc) const = 0;
			virtual void		DrawEllipse(const VWRectangle2D& rect) const = 0;

			// poly drawing
			virtual void		DrawPolyBegin() = 0;
			virtual void		DrawPoly_AddCornerVertex(const VWPoint2D& pt) = 0;
			virtual void		DrawPoly_AddBezierVertex(const VWPoint2D& pt) = 0;
			virtual void		DrawPoly_AddCubicVertex(const VWPoint2D& pt) = 0;
			virtual void		DrawPoly_AddArcVertex(const VWPoint2D& pt, double dRadius) = 0;
			virtual void		DrawPolyEnd() = 0;
			virtual void		DrawPolyEnd(const VWTransformMatrix& matrix) = 0;

			// object drawing
			virtual void		DrawObject(MCObjectHandle hObj, bool bDoSelect) const = 0;
			virtual void		DrawNurbsObject(MCObjectHandle hNurbsObj) const = 0;
			virtual void		DrawDataDisplayBar(bool bDrawFramesFirst = false) const = 0;

			// attributes
			virtual void		SetPenSize(short penSize = 0) const = 0;
			virtual void		SetPenPattern(short penPatIndex = 2) const = 0;
		};

		// ---------------------------------------------------------------------------
		class IToolPropertiesProvider
		{
		public:
			virtual				~IToolPropertiesProvider() {}
			virtual bool		SetProperty(long propID, bool bValue) = 0;
			virtual bool		SetPropertyChar(long propID, char value) = 0;
		};
	}

	// Helper tools
	namespace PluginSupport
	{
		//------------------------------------------------------------------
		// Registeres a public VCOM interface implementation -- available in entire Vectorworks
		template<class T> void REGISTER_Interface(const VWIID& iid, long action, void* pInfo, long& ioData, CallBackPtr cbp, long& reply)
		{
			if ( action == kVCOMRegisterInterfaces ) {
				::GS_VWRegisterInterface(iid, pInfo);
			}
			else if ( action == kVCOMQueryInterface && pInfo ) {
				if ( *((VWIID*) pInfo) == iid ) {
					T*			instance	= new T( cbp );
					ioData					= (long) instance;
				}
				if ( ioData == 0 || reply != kVCOMError_NoError ) {
					reply	= kVCOMError_NoInstance;
				}
			}
			else if ( action == kVCOMReleaseInterface && pInfo ) {
				if ( *((VWIID*) pInfo) == iid ) {
					T* implementation = dynamic_cast<T*>( (IVWUnknown*) ioData );
					if ( VERIFYN( kVStanev, implementation ) ) {
						delete implementation;
					}
					if ( implementation == 0 || reply != kVCOMError_NoError ) {
						reply	= kVCOMError_NoInstance;
					}
				}
			}
			else if ( action == kVCOMQueryInterfaceRTTI ) {
				if ( *((VWIID*) pInfo) == iid ) {
				}
			}
		}

		//------------------------------------------------------------------
		// Registeres a public VCOM interface implementation -- available in entire Vectorworks
		template<class T> void REGISTER_Interface(const VWGroupID& iidGroup, const VWIID& iid, long action, void* pInfo, long& ioData, CallBackPtr cbp, long& reply)
		{
			if ( action == kVCOMRegisterInterfaces ) {
				::GS_VWRegisterInterface(iid, iidGroup, pInfo);
			}
			else if ( action == kVCOMQueryInterface && pInfo ) {
				if ( *((VWIID*) pInfo) == iid ) {
					T*			instance	= new T( cbp );
					ioData					= (long) instance;
				}
				if ( ioData == 0 ) {
					// preserve the error
					if ( reply == kVCOMError_NoError ) {
						reply	= kVCOMError_NoInstance;
					}
				}
			}
			else if ( action == kVCOMReleaseInterface && pInfo ) {
				if ( *((VWIID*) pInfo) == iid ) {
					T* implementation = dynamic_cast<T*>( (IVWUnknown*) ioData );
					if ( VERIFYN( kVStanev, implementation ) ) {
						delete implementation;
						implementation	= NULL;
					}
					else {
						// preserve the error
						if ( reply == kVCOMError_NoError ) {
							reply	= kVCOMError_NoInstance;
						}
					}
				}
			}
		}

		// --------------------------------------------------------------------------------------------------------
		// Registeres interface implementation available only through another interface
		template<class T> void REGISTER_HiddenInterface(const VWIID& iid, long action, void* pInfo, long& ioData, CallBackPtr cbp, long& reply)
		{
			if ( action == kVCOMRegisterInterfaces ) {
				;
			}
			else if ( action == kVCOMQueryInterface && pInfo ) {
				;
			}
			else if ( action == kVCOMReleaseInterface && pInfo ) {
				if ( *((VWIID*) pInfo) == iid ) {
					T* implementation = dynamic_cast<T*>( (IVWUnknown*) ioData );
					if ( VERIFYN( kVStanev, implementation ) ) {
						delete implementation;
					}
					if ( implementation == 0 || reply != kVCOMError_NoError ) {
						reply	= kVCOMError_NoInstance;
					}
				}
			}
		}

		// --------------------------------------------------------------------------------------------------------
		// Registeres extension application interface implementation
		template<class T> void REGISTER_Extension(const VWGroupID& groupID, long action, void* pInfo, long& ioData, CallBackPtr cbp, long& reply)
		{
			using namespace VectorWorks::Extension;
			if ( action == kVCOMRegisterInterfaces ) {
				::GS_VWRegisterInterface(T::_GetIID(), groupID, pInfo);
			}
			else if ( action == kVCOMQueryInterface && pInfo ) {
				if ( *((VWIID*) pInfo) == T::_GetIID() ) {
					T*			instance	= new T( cbp );
					// check if this is the only one interface recognized
					ASSERTN( kVStanev, ioData == 0 );
					ioData					= (long) instance;
					if ( ioData == 0 || reply != kVCOMError_NoError ) {
						reply	= kVCOMError_NoInstance;
					}
				}
			}
			else if ( action == kVCOMReleaseInterface && pInfo ) {
				if ( *((VWIID*) pInfo) == T::_GetIID() ) {
					T* implementation = dynamic_cast<T*>( (IVWUnknown*) ioData );
					if ( VERIFYN( kVStanev, implementation ) ) {
						delete implementation;
					}
					if ( implementation == 0 || reply != kVCOMError_NoError ) {
						reply	= kVCOMError_NoInstance;
					}
				}
			}
		}
	}
}


#undef DEFINE_VWExtension
#undef IMPLEMENT_VWExtension

#define		DEFINE_VWExtension																\
 protected:																					\
	virtual ILegacyEventSink*					CreateLegacyEventSink(IVWUnknown* parent);	\
 public:																					\
	static const VWIID&							_GetIID();									\
	virtual Sint32 VCOM_CALLTYPE				GetVersion();								\
	virtual const TXString& VCOM_CALLTYPE		GetUniversalName();							\
 private:																					\
	TXString	fUniverslalName;

#define		IMPLEMENT_VWExtension( TClass, TEventSkink, TName, TVer, TIID1, TIID2, TIID3, TIID4, TIID5, TIID6, TIID7, TIID8, TIID9, TIID10, TIID11) \
	ILegacyEventSink*				TClass::CreateLegacyEventSink(IVWUnknown* parent)	{ return new TEventSkink( parent ) ; }					\
	const VWIID&					TClass::_GetIID()									{ static VWIID iid = { TIID1, TIID2, TIID3, {TIID4, TIID5, TIID6, TIID7, TIID8, TIID9, TIID10, TIID11} }; return iid; }	\
	Sint32 VCOM_CALLTYPE			TClass::GetVersion()								{ return TVer; }										\
	const TXString& VCOM_CALLTYPE	TClass::GetUniversalName()							{ fUniverslalName = TName; fUniverslalName.AnsiToMac(); return fUniverslalName; }



#define		DEFINE_VWProtection		protected:																							\
									virtual IProviderProtection*	CreateProtectionSink(IVWUnknown* parent);							\
									virtual bool					GetProtectionSinkIID(TSinkIID& outIID);								\
									class CExtMenu_Protection : public VCOMImpl<IProviderProtection> { public:							\
											CExtMenu_Protection(IVWUnknown* parent, Uint16 magicNumber, Uint32 products, Uint16 modes)	\
												: VCOMImpl<IProviderProtection>( parent )												\
												{ fMagicNumber = magicNumber; fProducts = products; fModes = modes; }					\
											virtual Uint16	VCOM_CALLTYPE	GetMagicNumber()	{ return fMagicNumber;	}				\
											virtual Uint32	VCOM_CALLTYPE	GetProducts()		{ return fProducts; }					\
											virtual Uint16	VCOM_CALLTYPE	GetModes()			{ return fModes; }						\
										private: Uint16 fMagicNumber; Uint32 fProducts; Uint16 fModes;									\
									};

#define		IMPLEMENT_VWProtection( TClass, IID, MAGIC_NUM, PRODUCTS, MODES )						\
					bool TClass::GetProtectionSinkIID(TSinkIID& outIID) {							\
						outIID	= IID;																\
						return true;																\
					}																				\
					IProviderProtection* TClass::CreateProtectionSink(IVWUnknown* parent) {			\
						return new CExtMenu_Protection( parent, MAGIC_NUM, PRODUCTS, MODES );		\
					}

#ifndef _VWFC_LEGACY

#include "VWExtensionParametric.h"
#include "VWExtensionMenu.h"
#include "VWExtensionTool.h"
#include "VWExtensionVSFunctions.h"

#endif
