//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#include "StdHeaders.h"

#include "VWFC/PluginSupport/DirectModellingSupport.h"

using namespace VWFC::PluginSupport;

// ------------------------------------------------------------------------------------------------------
CParametricHotSpot::CParametricHotSpot()
{
	fHotspotID	= 0;
	fType		= kScreenPlanarSelBox;
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
	fCursorSpec = "";
#else
	fCursorID	= 0;
#endif
	fUserData	= 0;
}

CParametricHotSpot::CParametricHotSpot(HotSpotType type, const WorldPt3& location, const TXString& tipText, long data)
{
	fHotspotID	= 0;
	fType		= type;
	fLocation	= location;
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
	fCursorSpec = "";
#else
	fCursorID	= 0;
#endif
	fTipText	= tipText;
	fUserData	= data;
}

#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
CParametricHotSpot::CParametricHotSpot(HotSpotType type, const WorldPt3& location, const TXString& cursorSpec, const TXString& tipText, long data)
{
	fHotspotID	= 0;
	fType		= type;
	fLocation	= location;
	fCursorSpec	= cursorSpec;
	fTipText	= tipText;
	fUserData	= data;
}

#else

CParametricHotSpot::CParametricHotSpot(HotSpotType type, const WorldPt3& location, short cursorID, const TXString& tipText, long data)
{
	fHotspotID	= 0;
	fType		= type;
	fLocation	= location;
	fCursorID	= cursorID;
	fTipText	= tipText;
	fUserData	= data;
}
#endif

CParametricHotSpot::~CParametricHotSpot()
{
}

void CParametricHotSpot::Add()
{
	if ( fHotspotID != 0 ) {
		this->Remove();
	}
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
	fHotspotID	= gSDK->AddHotSpotN( fType, fLocation, fCursorSpec, fTipText, fUserData );
#else
	fHotspotID	= gSDK->AddHotSpotN( fType, fLocation, fCursorID, fTipText, fUserData );
#endif
}

void CParametricHotSpot::Remove()
{
	if ( fHotspotID != 0 ) {
		gSDK->RemoveHotSpot( fHotspotID );
	}
	fHotspotID	= 0;
}

bool CParametricHotSpot::IsAdded() const
{
	return fHotspotID != 0;
}

void CParametricHotSpot::Set(HotSpotType type, const WorldPt3& location, const TXString& tipText)
{
	fType		= type;
	fLocation	= location;
#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
	fCursorSpec.Clear();
#else
	fCursorID	= 0;
#endif
	fTipText	= tipText;
	if ( fHotspotID != 0 ) {
		this->Remove();
		this->Add();
	}
}

#if T00809_IMAGE_RESOURCE_MANAGER_ENABLED
void CParametricHotSpot::Set(HotSpotType type, const WorldPt3& location, const TXString& cursorSpec, const TXString& tipText)
{
	fType		= type;
	fLocation	= location;
	fCursorSpec	= cursorSpec;
	fTipText	= tipText;
	if ( fHotspotID != 0 ) {
		this->Remove();
		this->Add();
	}
}
#else
void CParametricHotSpot::Set(HotSpotType type, const WorldPt3& location, short cursorID, const TXString& tipText)
{
	fType		= type;
	fLocation	= location;
	fCursorID	= cursorID;
	fTipText	= tipText;
	if ( fHotspotID != 0 ) {
		this->Remove();
		this->Add();
	}
}
#endif

void CParametricHotSpot::SetLocation(const WorldPt3& location)
{
	fLocation	= location;
	if ( fHotspotID != 0 ) {
		this->Remove();
		this->Add();
	}
}

long CParametricHotSpot::GetHotspotID() const
{
	return fHotspotID;
}

const WorldPt3& CParametricHotSpot::GetLocation() const
{
	return fLocation;
}

long CParametricHotSpot::GetUserData() const
{
	return fUserData;
}

void CParametricHotSpot::SetUserData(long userData)
{
	fUserData	= userData;
}

// ------------------------------------------------------------------------------------------------------
CParametricCustomBar::CParametricCustomBar()
{
	fCustomBarID	= 0;
}

CParametricCustomBar::~CParametricCustomBar()
{
}

bool CParametricCustomBar::IsValid() const
{
	return fCustomBarID != 0;
}

long CParametricCustomBar::GetCustomBarID() const
{
	return fCustomBarID;
}

bool CParametricCustomBar::Create()
{
	ASSERTN( kVStanev, fCustomBarID == 0 );
	return gSDK->CustomBarCreate( DemoteTo<short>( kVStanev, farrFields.size() ), fCustomBarID );
}

void CParametricCustomBar::Release()
{
	if ( fCustomBarID != 0 ) {
		gSDK->CustomBarRelease( fCustomBarID );
	}
	fCustomBarID	= 0;
}

void CParametricCustomBar::Install()
{
	if ( VERIFYN( kVStanev, fCustomBarID != 0 ) ) {
		gSDK->CustomBarInstall( fCustomBarID );
	}
}

void CParametricCustomBar::Pause(bool beginPause)
{
	if ( beginPause ) {
		gSDK->CustomBarInstall( 0 );
	}
	else {
		if ( fCustomBarID != 0 ) {
			gSDK->CustomBarInstall( fCustomBarID );
		}
	}
}

void CParametricCustomBar::ClearFields()
{
	farrFields.clear();
}

void CParametricCustomBar::AddField(const TXString& field)
{
	CustomBarFieldInfo theField;
	theField.fIsEditable	= true;
	field.CopyInto( theField.fFieldLabel );

	farrFields.push_back( theField );
}

void CParametricCustomBar::SetUp()
{
	if ( this->IsValid() ) {
		size_t	fieldsCnt	= farrFields.size();
		for(size_t i=0; i<fieldsCnt; i++) {
			const CustomBarFieldInfo& fi	= farrFields[ i ];
			gSDK->CustomBarSetFieldInfo( fCustomBarID, DemoteTo<short>(kVStanev, i), fi );
		}
	}
}

void CParametricCustomBar::SetDown()
{
}

void CParametricCustomBar::SetFieldWorldCoord(size_t fieldIndex, WorldCoord value)
{
	if ( this->IsValid() ) {
		gSDK->CustomBarSetFieldWorldCoord( fCustomBarID, DemoteTo<short>(kVStanev, fieldIndex), value );
	}
}

void CParametricCustomBar::SetFieldLock(size_t fieldIndex, bool isLocked)
{
	if ( this->IsValid() ) {
		CustomBarFieldInfo fi;
		gSDK->CustomBarGetFieldInfo( fCustomBarID, DemoteTo<short>(kVStanev, fieldIndex), fi );
		fi.fIsLocked	= isLocked;
		gSDK->CustomBarSetFieldInfo( fCustomBarID, DemoteTo<short>(kVStanev, fieldIndex), fi );
	}
}


