//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#include "StdHeaders.h"

#if _MINICAD_
# include "../VWFC/PluginSupport/VWExtensions.h"
# include "../VWFC/PluginSupport/VWExtensionMenu.h"
#else
# include "VWFC/PluginSupport/VWExtensions.h"
# include "VWFC/PluginSupport/VWExtensionMenu.h"
#endif

using namespace VWFC::PluginSupport;

// --------------------------------------------------------------------------------------------------------
VWExtensionMenu::VWExtensionMenu(	CallBackPtr				cbp,
									const SMenuDef&			menuDef,
									const SMenuChunkDef*	arrMenuChunks,
									EMenuChunkType			chunkType)
	: fCBP( cbp )
	, fMenuDef( menuDef )
	, fLegacyEventSink( NULL )
	, fMenuChunkSink( NULL )
	, fProtectionObtained( false )
	, fHasProtection( false )
	, fProtectionSink( NULL )
	, fMenuChunkType( chunkType )
	, fMenuChunkIsDirty( false )
{
	gCBP			= fCBP;
	fstrTitle.Load( fMenuDef.fTitle.fListID, fMenuDef.fTitle.fIndex );
	fstrCategory.Load( fMenuDef.fCategory.fListID, fMenuDef.fCategory.fIndex );
	if ( fMenuDef.fHelpText.fID > 0 ) {
		VERIFYN( kVStanev, VWFC::Tools::CSTRResLoader::Instance().LoadText( fMenuDef.fHelpText.fID, fstrHelp ) );
	}

	const SMenuChunkDef* currChunk	= arrMenuChunks;
	while( currChunk && currChunk->fItemText.fListID && currChunk->fItemText.fIndex ) {
		SChunkInfo	item;
		item.fItem				= TXString( currChunk->fItemText.fListID, currChunk->fItemText.fIndex );
		if ( currChunk->fContextualHelpID ) {
			item.fContextualHelpID	= currChunk->fContextualHelpID;
		}

		farrSubItems.push_back( item );

		currChunk++;		
	}

	if ( menuDef.fContextualHelpID ) {
		fstrContextualHelpID	= menuDef.fContextualHelpID;
	}
}

VWExtensionMenu::~VWExtensionMenu()
{
}

void VWExtensionMenu::OnRefCountZero()
{
	if ( fLegacyEventSink ) {
		delete fLegacyEventSink;
		fLegacyEventSink	= NULL;
	}

	if ( fMenuChunkSink ) {
		delete fMenuChunkSink;
		fMenuChunkSink	= NULL;
	}

	if ( fProtectionSink ) {
		delete fProtectionSink;
		fProtectionSink	= NULL;
	}

	// immediatelly delete this instance
	::GS_VWNotifyDeleteInterface( this );
}


IEventSink* VWExtensionMenu::QueryEventSink(const TSinkIID& iid)
{
	if ( fProtectionObtained == false ) {
		fHasProtection			= this->GetProtectionSinkIID( fProtectionSinkIID );
		fProtectionObtained		= true;
	}

	IEventSink*	pResult	= NULL;
	if ( IID_LegacyEventSink == iid ) {
		if ( fLegacyEventSink == NULL ) {
			fLegacyEventSink	= this->CreateLegacyEventSink( this );
			if ( fLegacyEventSink ) {
				fLegacyEventSink->AddRef();
				VWMenu_EventSink*	vwfcSink	= dynamic_cast<VWMenu_EventSink*>( fLegacyEventSink );
				if ( vwfcSink ) {
					vwfcSink->SetUniversalName( this->GetUniversalName() );
				}
			}
		}
		pResult	= fLegacyEventSink;
	}
	else if ( IID_MenuChunk2Provider == iid ) {
		if ( farrSubItems.size() > 0 ) {
			if ( fMenuChunkSink == NULL ) {
				fMenuChunkSink	= new CMenuChunk( this );
			}

			if ( fMenuChunkSink ) {
				fMenuChunkSink->AddRef();
			}

			pResult	= fMenuChunkSink;
		}
	}
	else if ( fHasProtection && fProtectionSinkIID == iid ) {
		if ( fProtectionSink == NULL ) {
			fProtectionSink	= this->CreateProtectionSink( this );
			if ( fProtectionSink == NULL ) {
				fHasProtection	= false;
			}
		}

		if ( fProtectionSink ) {
			fProtectionSink->AddRef();
			pResult	= fProtectionSink;
		}
	}

	return pResult;
}

bool VWExtensionMenu::GetProtectionSinkIID(TSinkIID& outIID)
{
	return false;
}

IProviderProtection* VWExtensionMenu::CreateProtectionSink(IVWUnknown* parent)
{
	return NULL;
}

const TXString& VWExtensionMenu::GetTitle()
{
	return fstrTitle;
}

const TXString& VWExtensionMenu::GetCategory()
{
	return fstrCategory;
}

const TXString& VWExtensionMenu::GetHelpText()
{
	return fstrHelp;
}

const TXString& VWExtensionMenu::GetContextualHelpID(size_t chunkIndex)
{
	if ( chunkIndex == size_t(-1) ) {
		return fstrContextualHelpID;
	}
	else {
		if ( chunkIndex < farrSubItems.size() ) {
			const SChunkInfo&	chunkInfo	= farrSubItems[ chunkIndex ];
			return chunkInfo.fContextualHelpID;
		}
		else {
			return fstrContextualHelpID;
		}
	}
}

void VWExtensionMenu::GetNeedsFlags(Uint16& outNeeds, Uint16& outNeedsNot)
{
	outNeeds	= fMenuDef.fNeeds;
	outNeedsNot	= fMenuDef.fNeedsNot;
}

void VWExtensionMenu::GetVersionInfo(Sint16& outCreated, Sint16& outModified, Sint16& outRetired)
{
	outCreated	= fMenuDef.fVersionCreated;
	outModified	= fMenuDef.fVersoinModified;
	outRetired	= fMenuDef.fVersoinRetired;
}

void VWExtensionMenu::GetSubMenuItems(TSubItemsVector& outArrItems)
{
	outArrItems	= farrSubItems;
}

void VWExtensionMenu::SetSubMenuItems(const TSubItemsVector& arrItems)
{
	fMenuChunkIsDirty	= true;
	farrSubItems		= arrItems;
}

// --------------------------------------------------------------------------------------------------------
VWExtensionMenu::CMenuChunk::CMenuChunk(VWExtensionMenu* parent)
	: VCOMImpl<IMenuChunk2Provider>( parent )
	, fParent( parent )
{
}

VWExtensionMenu::CMenuChunk::~CMenuChunk()
{
}

size_t VWExtensionMenu::CMenuChunk::GetItemsCount()
{
	return fParent->farrSubItems.size();
}

const TXString& VWExtensionMenu::CMenuChunk::GetItemTitle(size_t index)
{
	ASSERTN( kVStanev, index < fParent->farrSubItems.size() );
	return fParent->farrSubItems[ index ].fItem;
}

const TXString& VWExtensionMenu::CMenuChunk::GetContextualHelpID(size_t index)
{
	ASSERTN( kVStanev, index < fParent->farrSubItems.size() );
	return fParent->farrSubItems[ index ].fContextualHelpID;
}

EMenuChunkType VWExtensionMenu::CMenuChunk::GetType()
{
	return fParent->fMenuChunkType;
}

const TXString& VWExtensionMenu::CMenuChunk::GetItemHelp(size_t index)
{
	ASSERTN( kVStanev, index < fParent->farrSubItems.size() );
	return fParent->farrSubItems[ index ].fHelpText;
}

bool VWExtensionMenu::CMenuChunk::IsChunkDirty()
{
	return fParent->fMenuChunkIsDirty;
}

void VWExtensionMenu::CMenuChunk::ClearChunkDirty()
{
	fParent->fMenuChunkIsDirty	= false;
}

// --------------------------------------------------------------------------------------------------------
// predefine these hidden memory managment functions
namespace VWFC
{
	void	AllocateMemoryManagers(VWFC::MemoryManager::MManager** ppOutOldManager, VWFC::MemoryManager::MManager** ppOutOldVWManager);
	void	FreeMemoryManagers(VWFC::MemoryManager::MManager* pOldManager, VWFC::MemoryManager::MManager* pOldVWManager);
}

// --------------------------------------------------------------------------------------------------------
VWMenu_EventSink::VWMenu_EventSink(IVWUnknown* parent)
	: VCOMImpl<ILegacyEventSink>( parent )
{
	fpMenuChunkInfo		= NULL;
	fResult				= 0;
}

VWMenu_EventSink::~VWMenu_EventSink()
{
}

void VWMenu_EventSink::SetUniversalName(const TXString& name)
{
	fUnivName	= name;
}

long VWMenu_EventSink::LegacyMain(long action, long message1, long )
{
	long	reply = 0;

	// create the memory namagers
	// and verify their presents
	VWFC::MemoryManager::MManager* pOldManager		= NULL;
	VWFC::MemoryManager::MManager* pOldVWManager	= NULL;
	VWFC::AllocateMemoryManagers( & pOldManager, & pOldVWManager );

	VWFC_VERIFY( VWFC::GetMManager() != NULL && VWFC::GetVWMManager() != NULL );
	if ( VWFC::GetMManager() != NULL && VWFC::GetVWMManager() != NULL ) {

		switch(action) {
			case kMenuDoInterface:
				this->SetMenuChunkInfo( (MenuChunkInfo*) message1 );
				this->DoInterface();
				reply	= this->GetResult();
				break;

			case kMenuAddItems:
				this->OnDynamicVariableChunkUpdate();
				break;

			case kMenuCheckHilite:
				this->OnDynamicVariableChunkUpdate();
				break;

			case kMenuItemEnabled: {
					MenuChunkInfo	menuInfo;
					menuInfo.menuID			= 0;
					menuInfo.itemID			= 0;
					menuInfo.modifiers		= 0;
					menuInfo.chunkIndex		= DemoteTo<Sint16>( kVStanev, message1 );
					this->SetMenuChunkInfo( & menuInfo );

					bool	bIsItemEnabled	= this->GetItemEnabled();
					reply					= this->GetResult();
					if ( reply == 0 ) {
						reply	= bIsItemEnabled ? kMenuItemIsEnabled : kMenuItemIsDisabled;
					}
				} break;

			case kMenuAccessCheck:
				break;
			case kMenuItemCheck:
				break;

			case kMenuItemDisplayContextualHelp:
				DSTOP((kVStanev, "we should not be here for module plug-ins"));
				break;

			default:
				reply = this->OnDefaultEvent( action, message1 );
				break;

		}

		// dump local memory leaks
		if ( VWFC::GetMManager() ) {
			VWFC::GetMManager()->DumpMemory( fUnivName );
		}
	}
		
	// release the memory managers
	VWFC::FreeMemoryManagers( pOldManager, pOldVWManager );

	return reply;
}

void VWMenu_EventSink::DoInterface()
{
}

bool VWMenu_EventSink::GetItemEnabled()
{
	return true;
}

long VWMenu_EventSink::GetUserData() const
{
	return 0;
}

void VWMenu_EventSink::SetUserData(long /*userData*/)
{
}

long VWMenu_EventSink::GetResult() const
{
	return fResult;
}

void VWMenu_EventSink::SetResult(long result)
{
	fResult	= result;
}

void VWMenu_EventSink::SetMenuChunkInfo(MenuChunkInfo* pMenuChunkInfo)
{
	fpMenuChunkInfo		= pMenuChunkInfo;
}

long VWMenu_EventSink::OnDefaultEvent(long /*action*/, long /*message*/)
{
	return 0;
}

void VWMenu_EventSink::OnDynamicVariableChunkUpdate()
{
}
