//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#include "StdHeaders.h"

#include "EditTextCtrl.h"

#include "Dialog.h"

using namespace VWFC::VWUI;

VWEditTextCtrl::VWEditTextCtrl(TControlID id)
: VWControl( id )
{
}

VWEditTextCtrl::~VWEditTextCtrl()
{
}

bool VWEditTextCtrl::CreateControl(VWDialog* pDlg, const TXString& text, short widthInCharacters, short heightInLines/*=1*/)
{
	::GS_CreateEditText( gCBP, pDlg->GetControlID(), fControlID, text, widthInCharacters, heightInLines );
	return VWControl::CreateControl( pDlg );
}

void VWEditTextCtrl::GetText255(TXString& string)
{
	GSStr255	strArr;
	::GS_GetItemText( gCBP, fpParentDlg->GetControlID(), fControlID, strArr );
	string		= strArr;
}

void VWEditTextCtrl::GetMultilineText(TXString& string)
{
	GSHandle	hText	= ::GS_GetMultilineText( gCBP, fpParentDlg->GetControlID(), fControlID );
	if (VERIFYN(kTArdoin, hText != NULL))
	{
		size_t		size	= 0;
		::GS_GetHandleSize( gCBP, hText, size );

		// Note: GS_GetMultilineText returns a handle that is neither null terminated nor prefixed with a size. We have to add a null to the end.
		if (::GS_SetHandleSize(gCBP, hText, size + 1))
		{	// append a null character to our newly resized text handle
			char*		szText	= *hText;
			szText[size]		= 0x00;

			// finally, copy the text out
			string				= szText;
		}
		::GS_DisposeHandle( gCBP, hText );
	}
}

void VWEditTextCtrl::SetText255(const TXString& string)
{
	GSStr255	strArr;
	string.CopyInto( strArr );
	::GS_SetItemText( gCBP, fpParentDlg->GetControlID(), fControlID, strArr );
}

void VWEditTextCtrl::SetMultilineText(const TXString& string)
{
	size_t		len		= string.GetLength();
	GSHandle	hText	= ::GS_NewHandle( gCBP, len+1 );
	::GS_HLock( gCBP, hText );
	char*		szText	= *hText;
	string.CopyInto( szText/*, len*/ );
	::GS_HUnlock( gCBP, hText );

	::GS_SetMultilineText( gCBP, fpParentDlg->GetControlID(), fControlID, hText );

	::GS_DisposeHandle( gCBP, hText );
}

void VWEditTextCtrl::SelectContent()
{
	::GS_SelectEditText( gCBP, fpParentDlg->GetControlID(), fControlID );
}

void VWEditTextCtrl::DeselectContent()
{
	gSDK->DeselectEditText( fpParentDlg->GetControlID(), fControlID );
}
