//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//


#include "StdHeaders.h"

#include "TreeCtrlItem.h"

#include "Dialog.h"

using namespace VWFC::VWUI;

VWTreeCtrlItem	VWTreeCtrlContainer::kEmptyTreeItem;

// ---------------------------------------------------------------------------------------------------------------
VWTreeCtrlContainer::VWTreeCtrlContainer(TControlID dialogID, TControlID controlID, TTreeItemID itemID)
{
	fDialogID		= dialogID;
	fThisControlID	= controlID;
	fThisItemID		= itemID;
}

VWTreeCtrlContainer::VWTreeCtrlContainer(const VWTreeCtrlContainer& src)
{
	this->operator = ( src );
}

VWTreeCtrlContainer::~VWTreeCtrlContainer()
{
	size_t	cnt	= farrItems.size();
	for(size_t i=0; i<cnt; i++) {
		VWTreeCtrlItem*	item	= farrItems[ i ];
		if ( item ) {
			delete item;
		}
	}

	farrItems.clear();
}

VWTreeCtrlContainer& VWTreeCtrlContainer::operator=(const VWTreeCtrlContainer& src)
{
	fDialogID		= src.fDialogID;
	fThisControlID	= src.fThisControlID;
	fThisItemID		= src.fThisItemID;

	ASSERTN( kVStanev, false );
	return *this;
}

bool VWTreeCtrlContainer::operator==(const VWTreeCtrlContainer& src)
{
	return
			fDialogID		== src.fDialogID
		&&	fThisControlID	== src.fThisControlID
		&&	fThisItemID		== src.fThisItemID;
}

bool VWTreeCtrlContainer::operator!=(const VWTreeCtrlContainer& src)
{
	return ! this->operator == ( src );
}

bool VWTreeCtrlContainer::IsValid() const
{
	return (fDialogID != -1 && fThisControlID != -1);
}

VWTreeCtrlItem& VWTreeCtrlContainer::FindItem(TTreeItemID id)
{
	if ( this->IsValid() ) {
		size_t	cnt	= farrItems.size();
		for(size_t i=0; i<cnt; i++) {
			VWTreeCtrlItem*		item		= farrItems[ i ];
			if ( item ) {
				if ( id == item->fThisItemID ) {
					return *item;
				}

				VWTreeCtrlItem&	foundItem	= item->FindItem( id );
				if ( foundItem != VWTreeCtrlContainer::kEmptyTreeItem ) {
					// That's it! Found!
					return foundItem;
				}
			}
		}
	}

	return VWTreeCtrlContainer::kEmptyTreeItem;
}

size_t VWTreeCtrlContainer::GetChildItemsCount() const
{
	return farrItems.size();
}

VWTreeCtrlItem& VWTreeCtrlContainer::GetChildItemAt(size_t index) const
{
	if ( index < farrItems.size() ) {
		return *farrItems[ index ];
	}
	else {
		return VWTreeCtrlContainer::kEmptyTreeItem;
	}
}

bool VWTreeCtrlContainer::FindChildItemIndex(const VWTreeCtrlItem& itemToFind, VWTreeCtrlContainer*& outParentItem, size_t& outIndex)
{
	bool	found	= false;
	if ( this->IsValid() ) {
		size_t	cnt	= farrItems.size();
		for(size_t i=0; i<cnt && !found ; i++) {
			VWTreeCtrlItem*		item		= farrItems[ i ];
			if ( item ) {
				if ( itemToFind.GetItemID() == item->fThisItemID ) {
					outParentItem	= item;
					outIndex		= i;
					found			= true;
				}
				else {
					found	= item->FindChildItemIndex( itemToFind, outParentItem, outIndex );
				}
			}
		}
	}

	return found;
}

VWTreeCtrlItem& VWTreeCtrlContainer::AddChildItem(const TXString& text, bool addAtEnd)
{
	if ( ! this->IsValid() )
		return VWTreeCtrlContainer::kEmptyTreeItem;


	TTreeItemID		afterItemID	= -1;
	if ( addAtEnd && farrItems.size() > 0 ) {
		const VWTreeCtrlItem*	endItem	= farrItems[ farrItems.size() - 1 ];
		afterItemID						= endItem->fThisItemID;
	}

	TTreeItemID		newItemID	= gSDK->InsertTreeControlItem( fDialogID, fThisControlID, text, fThisItemID, afterItemID );
	VWTreeCtrlItem*	newItem		= new VWTreeCtrlItem( fDialogID, fThisControlID, newItemID );
	if ( ! addAtEnd ) {
		farrItems.insert( farrItems.begin(), newItem );
	}
	else {
		farrItems.push_back( newItem );
	}

	return *newItem;
}

VWTreeCtrlItem& VWTreeCtrlContainer::InsertChildItem(const TXString& text, const VWTreeCtrlItem& afterItem)
{
	if ( ! this->IsValid() )
		return VWTreeCtrlContainer::kEmptyTreeItem;

	TTreeItemID		afterItemID	= -1;
	if ( farrItems.size() > 0 ) {
		const VWTreeCtrlItem*	endItem	= farrItems[ farrItems.size() - 1 ];
		afterItemID						= endItem->fThisItemID;
	}

	TItemsArray::iterator	it	= farrItems.begin();
	for(; it != farrItems.end(); ++it)
	{
		const VWTreeCtrlItem*	theItem	= *it;
		if ( theItem && theItem->fThisItemID == afterItem.fThisItemID ) {
			afterItemID	= theItem->fThisItemID;
			break;
		}
	}

	TTreeItemID		newItemID	= gSDK->InsertTreeControlItem( fDialogID, fThisControlID, text, fThisItemID, afterItemID );
	VWTreeCtrlItem*	newItem		= new VWTreeCtrlItem( fDialogID, fThisControlID, newItemID );

	if ( it != farrItems.end() ) {
		farrItems.insert( it, newItem );
	}
	else {
		farrItems.push_back( newItem );
	}

	return *newItem;
}

void VWTreeCtrlContainer::DeleteItem()
{
	if ( this->IsValid() ) {
		VERIFYN( kVStanev, gSDK->RemoveTreeControlItem( fDialogID, fThisControlID, fThisItemID ) );

		TItemsArray::iterator	it	= farrItems.begin();
		for(; it != farrItems.end(); ++it)
		{
			const VWTreeCtrlItem*	theItem	= *it;
			if ( theItem && theItem->fThisItemID == fThisItemID ) {
				delete theItem;
				farrItems.erase( it );
				break;
			}
		}
	}
}

/*VWTreeCtrlItem& VWTreeCtrlContainer::GetParentItem()
{
}

VWTreeCtrlItem& VWTreeCtrlContainer::GetNextSiblingItem()
{
}

VWTreeCtrlItem& VWTreeCtrlContainer::GetPrevSiblingItem()
{
}*/

// ---------------------------------------------------------------------------------------------------------------
VWTreeCtrlItem::VWTreeCtrlItem()
	: VWTreeCtrlContainer( -1, -1, -1 )
{
}

VWTreeCtrlItem::VWTreeCtrlItem(TControlID dialogID, TControlID controlID, TTreeItemID itemID)
	: VWTreeCtrlContainer( dialogID, controlID, itemID )
{
}

VWTreeCtrlItem::VWTreeCtrlItem(const VWTreeCtrlItem& src)
	: VWTreeCtrlContainer( src )
{
}

VWTreeCtrlItem::~VWTreeCtrlItem()
{
}

VWTreeCtrlItem&	VWTreeCtrlItem::operator=(const VWTreeCtrlItem& src)
{
	((VWTreeCtrlContainer*)this)->operator = ( src );

	return *this;
}

bool VWTreeCtrlItem::operator==(const VWTreeCtrlItem& src)
{
	return ((VWTreeCtrlContainer*)this)->operator == ( src );
}

bool VWTreeCtrlItem::operator!=(const VWTreeCtrlItem& src)
{
	return ((VWTreeCtrlContainer*)this)->operator != ( src );
}

void VWTreeCtrlItem::ExpandItem(bool bExpand)
{
	if ( this->IsValid() ) {
		VERIFYN( kVStanev, gSDK->ExpandTreeControlItem( fDialogID, fThisControlID, fThisItemID, bExpand ) );
	}
}

void VWTreeCtrlItem::SelectItem()
{
	if ( this->IsValid() ) {
		VERIFYN( kVStanev, gSDK->SelectTreeControlItem( fDialogID, fThisControlID, fThisItemID ) );
	}
}

TTreeItemID VWTreeCtrlItem::GetItemID() const
{
	return fThisItemID;
}


void VWTreeCtrlItem::SetItemData(long data)
{
	if ( this->IsValid() ) {
		VERIFYN( kVStanev, gSDK->SetTreeControlItemData( fDialogID, fThisControlID, fThisItemID, data ) );
	}
}

long VWTreeCtrlItem::GetItemData() const
{
	if ( ! this->IsValid() )
		return 0;

	long	data	= 0;
	VERIFYN( kVStanev, gSDK->GetTreeControlItemData( fDialogID, fThisControlID, fThisItemID, data ) );
	return data;
}
